@@ -2,11 +2,11 @@
 
 import abc
 from dataclasses import dataclass, field
-from typing import Any, Callable, Dict, List, Optional, Type
+from typing import Any, Callable, Dict, List, Optional, Type, Union
 from urllib.parse import urlencode, urlparse
 
 import rasterio
-from geojson_pydantic.features import Feature
+from geojson_pydantic.features import Feature, FeatureCollection
 from morecantile import TileMatrixSet
 from rio_tiler.io import BaseReader, COGReader, MultiBandReader, MultiBaseReader
 from rio_tiler.models import Bounds, Info, Metadata
@@ -33,9 +33,9 @@ from titiler.core.models.mapbox import TileJSON
 from titiler.core.models.OGC import TileMatrixSetList
 from titiler.core.resources.enums import ImageType, MediaType, OptionalHeader
 from titiler.core.resources.responses import GeoJSONResponse, XMLResponse
-from titiler.core.utils import Timer, bbox_to_feature
+from titiler.core.utils import Timer, bbox_to_feature, data_stats
 
-from fastapi import APIRouter, Depends, Path, Query
+from fastapi import APIRouter, Body, Depends, Path, Query
 
 from starlette.requests import Request
 from starlette.responses import Response
@@ -146,6 +146,7 @@ class TilerFactory(BaseTilerFactory):
     # Add/Remove some endpoints
     add_preview: bool = True
     add_part: bool = True
+    add_statistics: bool = True
 
     def register_routes(self):
         """
@@ -172,6 +173,9 @@ class TilerFactory(BaseTilerFactory):
         if self.add_part:
             self.part()
 
+        if self.add_statistics:
+            self.statistics()
+
     ############################################################################
     # /bounds
     ############################################################################
@@ -411,7 +415,7 @@ class TilerFactory(BaseTilerFactory):
             tiles_url = self.url_for(request, "tile", **route_params)
 
             qs_key_to_remove = [
-                "TileMatrixSetIt",
+                "tilematrixsetid",
                 "tile_format",
                 "tile_scale",
                 "minzoom",
@@ -420,7 +424,7 @@ class TilerFactory(BaseTilerFactory):
             qs = [
                 (key, value)
                 for (key, value) in request.query_params._list
-                if key not in qs_key_to_remove
+                if key.lower() not in qs_key_to_remove
             ]
             if qs:
                 tiles_url += f"?{urlencode(qs)}"
@@ -482,18 +486,18 @@ class TilerFactory(BaseTilerFactory):
             tiles_url = self.url_for(request, "tile", **route_params)
 
             qs_key_to_remove = [
-                "TileMatrixSetIt",
+                "tilematrixsetid",
                 "tile_format",
                 "tile_scale",
                 "minzoom",
                 "maxzoom",
-                "SERVICE",
-                "REQUEST",
+                "service",
+                "request",
             ]
             qs = [
                 (key, value)
                 for (key, value) in request.query_params._list
-                if key not in qs_key_to_remove
+                if key.lower() not in qs_key_to_remove
             ]
             if qs:
                 tiles_url += f"?{urlencode(qs)}"
@@ -706,6 +710,190 @@ class TilerFactory(BaseTilerFactory):
 
             return Response(content, media_type=format.mediatype, headers=headers)
 
+        @self.router.post(
+            r"/crop", **img_endpoint_params,
+        )
+        @self.router.post(
+            r"/crop.{format}", **img_endpoint_params,
+        )
+        @self.router.post(
+            r"/crop/{width}x{height}.{format}", **img_endpoint_params,
+        )
+        def geojson_crop(
+            feature: Feature = Body(..., descriptiom="GeoJSON Feature."),
+            format: ImageType = Query(
+                None, description="Output image type. Default is auto."
+            ),
+            src_path=Depends(self.path_dependency),
+            layer_params=Depends(self.layer_dependency),
+            image_params=Depends(self.img_dependency),
+            dataset_params=Depends(self.dataset_dependency),
+            render_params=Depends(self.render_dependency),
+            colormap=Depends(self.colormap_dependency),
+            kwargs: Dict = Depends(self.additional_dependency),
+        ):
+            """Create image from a geojson feature."""
+            timings = []
+            headers: Dict[str, str] = {}
+
+            with Timer() as t:
+                with rasterio.Env(**self.gdal_config):
+                    with self.reader(src_path, **self.reader_options) as src_dst:
+                        data = src_dst.feature(
+                            feature.dict(exclude_none=True),
+                            **layer_params.kwargs,
+                            **image_params.kwargs,
+                            **dataset_params.kwargs,
+                            **kwargs,
+                        )
+                        dst_colormap = getattr(src_dst, "colormap", None)
+            timings.append(("dataread", round(t.elapsed * 1000, 2)))
+
+            with Timer() as t:
+                image = data.post_process(
+                    in_range=render_params.rescale_range,
+                    color_formula=render_params.color_formula,
+                )
+            timings.append(("postprocess", round(t.elapsed * 1000, 2)))
+
+            if not format:
+                format = ImageType.jpeg if data.mask.all() else ImageType.png
+
+            with Timer() as t:
+                content = image.render(
+                    add_mask=render_params.return_mask,
+                    img_format=format.driver,
+                    colormap=colormap or dst_colormap,
+                    **format.profile,
+                    **render_params.kwargs,
+                )
+            timings.append(("format", round(t.elapsed * 1000, 2)))
+
+            if OptionalHeader.server_timing in self.optional_headers:
+                headers["Server-Timing"] = ", ".join(
+                    [f"{name};dur={time}" for (name, time) in timings]
+                )
+
+            return Response(content, media_type=format.mediatype, headers=headers)
+
+    ############################################################################
+    # /statistics (Optional)
+    ############################################################################
+    def statistics(self):
+        """add statistics endpoints."""
+
+        @self.router.get(
+            "/statistics",
+            responses={
+                200: {
+                    "content": {"application/json": {}},
+                    "description": "Return dataset's statistics.",
+                }
+            },
+        )
+        def statistics(
+            src_path=Depends(self.path_dependency),
+            layer_params=Depends(self.layer_dependency),
+            image_params=Depends(self.img_dependency),
+            dataset_params=Depends(self.dataset_dependency),
+            categorical: bool = Query(
+                False, description="Return statistics for categorical dataset."
+            ),
+            c: List[Union[float, int]] = Query(
+                None, description="Pixels values for categories."
+            ),
+            p: List[int] = Query([2, 98], description="Percentile values."),
+            kwargs: Dict = Depends(self.additional_dependency),
+        ):
+            """Create image from a geojson feature."""
+            with rasterio.Env(**self.gdal_config):
+                with self.reader(src_path, **self.reader_options) as src_dst:
+                    data = src_dst.preview(
+                        **layer_params.kwargs,
+                        **image_params.kwargs,
+                        **dataset_params.kwargs,
+                        **kwargs,
+                    ).as_masked()
+
+            return data_stats(
+                data, categorical=categorical, categories=c, percentiles=p
+            )
+
+        @self.router.post(
+            "/statistics",
+            response_model=Union[Feature, FeatureCollection],
+            response_model_exclude_none=True,
+            response_class=GeoJSONResponse,
+            responses={
+                200: {
+                    "content": {"application/json": {}},
+                    "description": "Return dataset's statistics.",
+                }
+            },
+        )
+        def geojson_statistics(
+            features: Union[FeatureCollection, Feature] = Body(
+                ..., descriptiom="GeoJSON Feature or FeatureCollection."
+            ),
+            src_path=Depends(self.path_dependency),
+            layer_params=Depends(self.layer_dependency),
+            image_params=Depends(self.img_dependency),
+            dataset_params=Depends(self.dataset_dependency),
+            categorical: bool = Query(
+                False, description="Return statistics for categorical dataset."
+            ),
+            c: List[Union[float, int]] = Query(
+                None, description="Pixels values for categories."
+            ),
+            p: List[int] = Query([2, 98], description="Percentile values."),
+            kwargs: Dict = Depends(self.additional_dependency),
+        ):
+            """Create image from a geojson feature."""
+            if isinstance(features, FeatureCollection):
+                feat = []
+                for feature in features:
+                    with rasterio.Env(**self.gdal_config):
+                        with self.reader(src_path, **self.reader_options) as src_dst:
+                            data = src_dst.feature(
+                                feature.dict(exclude_none=True),
+                                **layer_params.kwargs,
+                                **image_params.kwargs,
+                                **dataset_params.kwargs,
+                                **kwargs,
+                            ).as_masked()
+
+                        feature.properties.update(
+                            {
+                                "statistics": data_stats(
+                                    data,
+                                    categorical=categorical,
+                                    categories=c,
+                                    percentiles=p,
+                                )
+                            }
+                        )
+                        feat.append(feature)
+                return FeatureCollection(features=feat)
+            else:
+                with rasterio.Env(**self.gdal_config):
+                    with self.reader(src_path, **self.reader_options) as src_dst:
+                        data = src_dst.feature(
+                            features.dict(exclude_none=True),
+                            **layer_params.kwargs,
+                            **image_params.kwargs,
+                            **dataset_params.kwargs,
+                            **kwargs,
+                        ).as_masked()
+
+                features.properties.update(
+                    {
+                        "statistics": data_stats(
+                            data, categorical=categorical, categories=c, percentiles=p,
+                        )
+                    }
+                )
+                return features
+
 
 @dataclass
 class MultiBaseTilerFactory(TilerFactory):
